% ----------------------
% SS.m
% ----------------------
% computes steady state and fills the values for year 't'
% in:   DF,w,taxinstrument
% out:  resid
% ----------------------

function resid = SS(inputs)

global t rho g G gam sigma psi
global Y Div w theta I J V K H A Omega Q C TB DF edy r
global A_0 alpha LS LD edl ell N NB delta
global epsl parl0 parl1 parp0 epsp hcut part ybar
global tauW tauF taul tauprof tauprofdeduct tauC b
global pc TaxF Exp Rev PB DG cG CG DGtarget edg budget_bal
global DF0 w0 tauW0 tauF0 tauprof0 taul0 cG0 eda edv tauC0

DF(t)    = DF0*inputs(1);
w(t)     = w0*inputs(2);

% budget-instrument choice
if budget_bal == 1; tauW(t) = tauW0*inputs(3); end
if budget_bal == 2; tauF(t) = tauF0*inputs(3); end
if budget_bal == 3; tauC(t) = tauC0*inputs(3); end
if budget_bal == 4; taul(t) = taul0*inputs(3); end
if budget_bal == 5; tauprof(t) = tauprof0*inputs(3); end
if budget_bal == 6; cG(t) = cG0*inputs(3); end

DG(t)    = DGtarget(t); % debt-rule

pc(t)    = 1+tauC(t);

% DEMOGRAPHY
N(t)     = NB(t)/(1-gam(t));
CG(t)    = cG(t)*N(t);

% HOURS SUPPLY
ell(t)   = (((1-tauW(t))*w(t)*theta(t)/pc(t))/parl0)^epsl;
dis_ell  = parl0*(epsl/(1+epsl))*ell(t)^((1+epsl)/epsl)-parl1;

% PARTICIPATION
hcut(t)  = ((1-tauW(t))*w(t)*ell(t)*theta(t)-b(t))/pc(t)-dis_ell;
part(t)  = 1-hcut(t)^(-epsp)*parp0^epsp;
hexp     = 1/part(t)*epsp/(1+epsp)*(parp0^epsp*hcut(t)^(1-epsp)-parp0);
dis_tot  = part(t)*dis_ell-(1-part(t))*hexp;

% TOTAL LABOR SUPPLY
LS(t)    = part(t)*ell(t)*theta(t)*N(t);

% PRODUCTION
uck      = (r(t)+delta*(1-tauprofdeduct(t)*tauprof(t)))/(1-tauprof(t));
K(t)     = LS(t)*(alpha*A_0/uck)^(1/(1-alpha));
Y(t)     = A_0*K(t)^alpha*LS(t)^(1-alpha);
LD(t)    = Y(t)*(1-alpha)/((1+tauF(t))*w(t));

% INVESTMENT
I(t)     = (delta+g)*K(t);
J(t)     = 1/2*psi*K(t)*(I(t)/K(t)-delta-g)^2;

% FIRM VALUE
TaxF(t)  = tauprof(t)*(Y(t)-(1+tauF(t))*w(t)*LD(t)-tauprofdeduct(t)*(J(t)+delta*K(t)));
Div(t)   = Y(t)-(1+tauF(t))*w(t)*LD(t)-I(t)-J(t)-TaxF(t);
V(t)     = (1+r(t))/(r(t)-g)*Div(t);

% CONSUMPTION AND SAVINGS
ybar(t)  = (1-tauW(t))*w(t)*LS(t)+(1-part(t))*b(t)*N(t)-taul(t)*N(t)-dis_tot*pc(t)*N(t);
H(t)     = (1+r(t))/(r(t)-g*gam(t)+1-gam(t))*ybar(t);
Omega(t) = 1/(1-(1/(1+rho))^sigma*((1+r(t))*pc(t)/pc(t))^(sigma-1)*gam(t));
Q(t)     = (DF(t)+DG(t)+V(t)+H(t))/(Omega(t)*pc(t));
C(t)     = Q(t)+dis_tot*N(t);
A(t)     = (1+r(t))/(r(t)-g)*(pc(t)*Q(t)-ybar(t));

% TRADE BALANCE AND FOREIGN ASSETS
TB(t)    = -DF(t)*(r(t)-g)/(1+r(t));

% GOVERNMENT
Exp(t)   = CG(t)+(1-part(t))*b(t)*N(t);
Rev(t)   = TaxF(t)+(tauF(t)*LD(t)+tauW(t)*LS(t))*w(t)+taul(t)*N(t)+tauC(t)*C(t);
PB(t)    = DG(t)*(r(t)-g)/(1+r(t));

% EXCESS DEMANDS
edy(t)   = I(t)+J(t)+C(t)+CG(t)+TB(t)-Y(t);
edl(t)   = LD(t)-LS(t);
edg(t)   = Rev(t)-Exp(t)-PB(t);
eda(t)   = DF(t)+DG(t)+V(t)-A(t);

% WALRAS LAW
edv(t)   = edy(t)+w(t)*edl(t)+(r(t)-g)/(1+r(t))*eda(t)+edg(t);
if abs(edv(t))> 1e-10; error('Error in SS.m: Walras Law does not hold!'); end

resid(1,1) = edy(t);
resid(2,1) = edl(t);
resid(3,1) = edg(t);

end