% ----------------------
% TE.m
% ----------------------
% computes the temporary equilibrium for the global time 't'
% in:   w,taxinstrument
% out:  resid
% ----------------------

function resid = TE(inputs)

global t rho g G sigma gam psi
global Y Div w theta I J V K H Omega Q C TB DF A edy r
global A_0 alpha LS LD edl ell N NB delta
global epsl parl0 parl1 parp0 epsp hcut part ybar
global tauW tauF taul tauprof tauprofdeduct tauC b
global pc TaxF Exp Rev PB DG cG CG DGtarget edg budget_bal
global w0 tauW0 tauF0 tauprof0 taul0 cG0 eda edv tauC0 DG0

w(t)    = w0*inputs(1);

% budget-instrument choice
if budget_bal == 1; tauW(t) = tauW0*inputs(2); end
if budget_bal == 2; tauF(t) = tauF0*inputs(2); end
if budget_bal == 3; tauC(t) = tauC0*inputs(2); tauC(t+1) = tauC(t); end
if budget_bal == 4; taul(t) = taul0*inputs(2); end
if budget_bal == 5; tauprof(t) = tauprof0*inputs(2); end
if budget_bal == 6; cG(t) = cG0*inputs(2); end

duration_no_debt_rule = 10; % consolidate over same time length
consoldistion_start   = 11;

if (t<=duration_no_debt_rule)
    % have no debt rule
elseif (t<=consoldistion_start)
    DG(t+1) = DG(t); % constant debt-rule
elseif (t<=(consoldistion_start+duration_no_debt_rule))
    DG(t+1) = DG(t)-(DG(consoldistion_start)-DG0)/duration_no_debt_rule;    % consolidation
elseif (t>consoldistion_start+duration_no_debt_rule)
    DG(t+1) = DG(t); % constant debt-rule
end   

pc(t)       = 1+tauC(t);
pc(t+1)     = 1+tauC(t+1);
CG(t)       = cG(t)*N(t);

% DEMOGRAPHY
N(t+1)  = gam(t+1)*N(t)+NB(t+1);

% HOURS SUPPLY
ell(t)   = (((1-tauW(t))*w(t)*theta(t)/pc(t))/parl0)^epsl;
dis_ell  = parl0*(epsl/(1+epsl))*ell(t)^((1+epsl)/epsl)-parl1;

% PARTICIPATION
hcut(t)  = ((1-tauW(t))*w(t)*ell(t)*theta(t)-b(t))/pc(t)-dis_ell;
part(t)  = 1-hcut(t)^(-epsp)*parp0^epsp;
hexp     = 1/part(t)*epsp/(1+epsp)*(parp0^epsp*hcut(t)^(1-epsp)-parp0);
dis_tot  = part(t)*dis_ell-(1-part(t))*hexp;

% TOTAL LABOR SUPPLY
LS(t)    = part(t)*ell(t)*theta(t)*N(t);

% PRODUCTION
Y(t)    = A_0*K(t)^alpha*LS(t)^(1-alpha);
LD(t)   = ((1-alpha)*A_0/((1+tauF(t))*w(t)))^(1/alpha)*K(t);

% INVESTMENT
psi2    = (1-tauprofdeduct(t)*tauprof(t))*psi;
a1      = psi2/K(t);
a2      = (1-(delta+g)*psi2)+(1-delta)*psi2;
a3      = (1-delta)*(1-(delta+g)*psi2)*K(t)-G*V(t+1)/(1+r(t+1));
I(t)    = (-a2+sqrt(a2^2-4*a1*a3))/(2*a1);
J(t)    = 1/2*psi*K(t)*(I(t)/K(t)-delta-g)^2;
K(t+1)  = (I(t)+(1-delta)*K(t))/G;

% FIRM VALUE
TaxF(t)  = tauprof(t)*(Y(t)-(1+tauF(t))*w(t)*LD(t)-tauprofdeduct(t)*(J(t)+delta*K(t)));
Div(t)   = Y(t)-(1+tauF(t))*w(t)*LD(t)-I(t)-J(t)-TaxF(t);
V(t)     = Div(t)+G*V(t+1)/(1+r(t+1));

% CONSUMPTION AND SAVINGS
Omega(t)= 1+(1/(1+rho))^sigma*((1+r(t+1))*pc(t)/pc(t+1))^(sigma-1)*gam(t+1)*Omega(t+1);
ybar(t) = (1-tauW(t))*w(t)*LS(t)+(1-part(t))*b(t)*N(t)-taul(t)*N(t)-dis_tot*pc(t)*N(t);
H(t)    = ybar(t)+gam(t+1)*G*H(t+1)*(N(t)/N(t+1))/(1+r(t+1));
A(t)    = DF(t)+DG(t)+V(t);
Q(t)    = (H(t)+A(t))/(Omega(t)*pc(t));
C(t)    = Q(t)+dis_tot*N(t);
A(t+1)  = (1+r(t))*(A(t)+ybar(t)-pc(t)*Q(t))/G;

% TRADE BALANCE AND FOREIGN ASSETS
TB(t)   = Y(t)-C(t)-I(t)-J(t)-CG(t);
DF(t+1) = (1+r(t))*(DF(t)+TB(t))/G;

% GOVERNMENT
Exp(t)   = CG(t)+(1-part(t))*b(t)*N(t);
Rev(t)   = TaxF(t)+(tauF(t)*LD(t)+tauW(t)*LS(t))*w(t)+taul(t)*N(t)+tauC(t)*C(t);

% LET DEBT EVOLVE FREELY FIRST 20 YEARS
if (t <= duration_no_debt_rule)
    PB(t)    = Rev(t)-Exp(t);
    DG(t+1)  = (DG(t)-PB(t))*(1+r(t+1))/G;
else
    PB(t)    = DG(t)-G*DG(t+1)/(1+r(t+1));
end

% EXCESS DEMANDS
edy(t)  = I(t)+J(t)+C(t)+CG(t)+TB(t)-Y(t);
edl(t)  = LD(t)-LS(t);
edg(t)  = Rev(t)-Exp(t)-PB(t);
eda(t)  = DF(t)+DG(t)+V(t)-A(t);
eda(t+1)= DF(t+1)+DG(t+1)+V(t+1)-A(t+1);

% WALRAS LAW
edv(t)  = edy(t)+w(t)*edl(t)+edg(t)+eda(t)-G*eda(t+1)/(1+r(t+1));
if abs(edv(t))> 1e-10; error('Error in TE.m: Walras Law does not hold!'); end

resid(1,1) = edl(t);

% LET DEBT EVOLVE FREELY FIRST 20 YEARS
if (t <= duration_no_debt_rule)
    resid(2,1) = inputs(2)-1;
else    
    resid(2,1) = edg(t);
end

end

